/*---------------------------------------------------------------------------*
 *                                   IT++			             *
 *---------------------------------------------------------------------------*
 * Copyright (c) 1995-2002 by Tony Ottosson, Thomas Eriksson, Pl Frenger,   *
 * Tobias Ringstrm, and Jonas Samuelsson.                                   *
 *                                                                           *
 * Permission to use, copy, modify, and distribute this software and its     *
 * documentation under the terms of the GNU General Public License is hereby *
 * granted. No representations are made about the suitability of this        *
 * software for any purpose. It is provided "as is" without expressed or     *
 * implied warranty. See the GNU General Public License for more details.    *
 *---------------------------------------------------------------------------*/

/*! 
  \file 
  \brief Definitions of Filter classes and functions
  \author Hkan Eriksson and Thomas Eriksson

  1.12

  2003/01/04 00:21:54 
*/
 
#ifndef __filter_h
#define __filter_h

#include <complex>

#include "base/itassert.h"
#include "base/vec.h"

/*! \defgroup filters Filter Classes and Functions
 */

/*! 
  \brief Virtual Filter Base Class.
  \ingroup filters

  The class is templated as follows: 
  <ul>
  <li> \c T1 is the type of the input samples</li>
  <li> \c T2 is the type of the filter coefficients</li>
  <li> \c T3 is the type of the output samples</li>
  </ul>
 */
template <class T1, class T2, class T3>
class Filter {
 public:
  //! Default constructor
  Filter();
  //! Filter a single sample.
  virtual T3 operator()(T1 Sample) { return filter(Sample); }
  //! Filter a vector.
  virtual Vec<T3> operator()(const Vec<T1> &v);
 protected:
  /*!
    \brief Pure virtual filter function. This is where the real filtering is done. Implement this function to create a new filter. 
  */
  virtual T3 filter(T1 Sample)=0;
};

/*! 
  \brief Moving Average Filter Base Class.
  \ingroup filters
  
  This class implements a moving average (MA) filter 
  according to
  \f[ 
  y(n) = coeff(0)*x(n) + coeff(1)*x(n-1) + ... + coeff(N)*x(n-N) 
  \f]  
  where \a coeff is the filter coefficients, \a x is the input and \a y is the output.  
  
  When filtering a vector, the lenght of the output vector equals the length of the input vector. 
  Internal states are kept in a filter memory. The first time the filter is used the internal states 
  have been set to zero.
  
  The class is templated as follows: 
  <ul>
  <li> \c T1 is the type of the input samples</li>
  <li> \c T2 is the type of the filter coefficients</li>
  <li> \c T3 is the type of the output samples</li>
  </ul>
*/
template <class T1, class T2, class T3>
class MA_Filter : public Filter<T1,T2,T3> {
 public:
  //! Class default constructor
  explicit MA_Filter();
  //! Class constructor setting the coefficients in the filter
  explicit MA_Filter(const Vec<T2> &Par);
  //! Class destructor
  virtual ~MA_Filter() { }
  //! Filter coefficient access function
  Vec<T2> get_coeffs() const { return coeffs; }
  //! Set the filter coefficients
  void set_coeffs(const Vec<T2> &c);
  //! Clears the filter memory
  void clear() { mem.clear(); }

//protected:
 private:
  virtual T3 filter(T1 Sample);

  Vec<T3> mem;
  Vec<T2> coeffs;
  long inptr;
};

/*! 
  \brief Autoregressive (AR) Filter Base Class.
  \ingroup filters
  
  This class implements a moving average (MA) filter 
  according to
  \f[
  coeff(0)*y(n) = x(n) - coeff(1)*y(n-1) - ... - coeff(N)*y(n-N)
  \f]
  where \a coeff is the filter coefficients, \a x is the input and \a y is the output.  
  
  When filtering a vector, the lenght of the output vector equals the length of the input vector. 
  Internal states are kept in a filter memory. The first time the filter is used the internal states 
  have been set to zero.
  
  The class is templated as follows: 
  <ul>
  <li> \c T1 is the type of the input samples</li>
  <li> \c T2 is the type of the filter coefficients</li>
  <li> \c T3 is the type of the output samples</li>
  </ul>
*/
template <class T1, class T2, class T3>
class AR_Filter : public Filter<T1,T2,T3> {
 public:
  //! Class constructor
  explicit AR_Filter();
  //! Class constructor setting the coefficnebts in the filter
  explicit AR_Filter(const Vec<T2> &Par);
  //! Class destructor
  virtual ~AR_Filter() { }
  //! Filter coefficient access function
  Vec<T2> get_coeffs() const { return coeffs; }
  //! Set the filter coefficients (and order)
  void set_coeffs(const Vec<T2> &c);
  //! Clears the filter memory
  void clear() { mem.clear(); }

//protected:
 private:
  virtual T3 filter(T1 Sample);

  Vec<T3> mem;
  Vec<T2> coeffs;
  long inptr;
};

/*! 
  \brief ARMA filter (Auto-Regressive Moving Average)
  \ingroup filters

  This function is templated as follows: \c T1 is the type of the input samples, 
  \c T2 is the type of the filter coefficients, and \c T3 is the type of the output samples.
*/
template <class T1, class T2, class T3>
Vec<T3> filter(const Vec<T2> &b, const Vec<T2> &a, const Vec<T1> &input);

/*! 
  \brief FIR MA filter (Finite Impulse Response Moving Average)
  \ingroup filters

  This function is templated as follows: \c T1 is the type of the input samples, 
  \c T2 is the type of the filter coefficients, and \c T3 is the type of the output samples.
*/
template <class T1, class T2, class T3>
Vec<T3> filter(const Vec<T2> &b, int one, const Vec<T1> &input);

/*! 
  \brief IIR AR filter (Infinite Impulse Response Auto-Regressive)
  \ingroup filters

  This function is templated as follows: \c T1 is the type of the input samples, 
  \c T2 is the type of the filter coefficients, and \c T3 is the type of the output samples.
*/
template <class T1, class T2, class T3>
Vec<T3> filter(int one, const Vec<T2> &a, const Vec<T1> &input);

/*! 
  \brief Design a Nth order FIR filter with cut-off frequency \c cutoff using the window method.
  \ingroup filters
*/
vec fir1(long N, double cutoff);

#endif // __filter_h
