/*---------------------------------------------------------------------------*
 *                                   IT++			             *
 *---------------------------------------------------------------------------*
 * Copyright (c) 1995-2002 by Tony Ottosson, Thomas Eriksson, Pl Frenger,   *
 * Tobias Ringstrm, and Jonas Samuelsson.                                   *
 *                                                                           *
 * Permission to use, copy, modify, and distribute this software and its     *
 * documentation under the terms of the GNU General Public License is hereby *
 * granted. No representations are made about the suitability of this        *
 * software for any purpose. It is provided "as is" without expressed or     *
 * implied warranty. See the GNU General Public License for more details.    *
 *---------------------------------------------------------------------------*/

/*! 
  \file 
  \brief Matrix Class Definitions 
  \author Tony Ottosson and Tobias Ringstrom

  1.13

  2003/01/04 00:21:54
*/

#ifndef __mat_h
#define __mat_h

#include <iostream>
#include <string>
#include <cstdlib>
#include <cstring>
#include "itconfig.h"
#include "base/itassert.h"

template<class T> class Vec;
template<class T> class Mat;
class bin;

// ----------------------------- Mat Friends -----------------------------

//! Horizontal concatenation of two matrices
template<class T> Mat<T> concat_horizontal(const Mat<T> &m1, const Mat<T> &m2);
//! Vertical concatenation of two matrices
template<class T> Mat<T> concat_vertical(const Mat<T> &m1, const Mat<T> &m2);
//! Addition of two matricies
template<class T> Mat<T> operator+(const Mat<T> &m1, const Mat<T> &m2);
//! Addition of a matrix and a scalar
template<class T> Mat<T> operator+(const Mat<T> &m, T t);
//! Addition of a scalar and a matrix
template<class T> Mat<T> operator+(T t, const Mat<T> &m);
//! Subtraction of two matrices
template<class T> Mat<T> operator-(const Mat<T> &m1, const Mat<T> &m2);
//! Subtraction of matrix and scalar
template<class T> Mat<T> operator-(const Mat<T> &m, T t);
//! Subtraction of scalar and matrix
template<class T> Mat<T> operator-(T t, const Mat<T> &m);
//! Negation of matrix
template<class T> Mat<T> operator-(const Mat<T> &m);
//! Multiplication of two matricies
template<class T> Mat<T> operator*(const Mat<T> &m1, const Mat<T> &m2);
//! Multiplication of matrix and vector
template<class T> Vec<T> operator*(const Mat<T> &m, const Vec<T> &v);
//! Multiplication of vector and matrix
template<class T> Vec<T> operator*(const Vec<T> &v, const Mat<T> &m);
//! Multiplication of matrix and scalar
template<class T> Mat<T> operator*(const Mat<T> &m, T t);
//! Multiplication of scalar and matrix 
template<class T> Mat<T> operator*(T t, const Mat<T> &m);
//! Element wise multiplication of two matricies
template<class T> Mat<T> elem_mult(const Mat<T> &m1, const Mat<T> &m2);
//! Division of matrix and scalar
template<class T> Mat<T> operator/(const Mat<T> &m, T t);
//! Element wise division of two matricies
template<class T> Mat<T> elem_div(const Mat<T> &m1, const Mat<T> &m2);

/*! 
  \brief Templated Matrix Class
  \author Tony Ottosson and Tobias Ringstrom
  
  Matrices can be of arbitrarily types, but conversions and functions are
  prepared for \c bin, \c short, \c int, \c double, and \c double_complex
  vectors and these are predefined as: \c bmat, \c smat, \c imat, \c mat,
  and \c cmat. \c double and \c double_complex are usually \c double and
  \c complex<double> respectively. However, this can be changed when \
  compiling the it++ (see installation notes for more details).

  Examples:

  Matrix Constructors:
  When constructing a matrix without a dimensions (memory) use
  \code mat temp; \endcode
  For construction of a matrix of a given size use
  \code mat temp(rows, cols); \endcode
  It is also possible to assign the constructed matrix the value and dimension
  of another matrix by
  \code vec temp(inmatrix); \endcode
  If you have explicit values you would like to assign to the matrix it is
  possible to do this using strings as:
  \code
  mat a("0 0.7;5 9.3"); // that is a = [0, 0.7; 5, 9.3]
  mat a="0 0.7;5 9.3";  // the constructor are called implicitly
  \endcode
  It is also possible to change dimension by
  \code temp.set_size(new_rows, new_cols, false); \endcode
  where \c false is used to indicate that the old values in \c temp
  is not copied. If you like to preserve the values use \c true.

  There are a number of methods to access parts of a matrix. Examples are
  \code
  a(5,3);     // Element number (5,3)
  a(5,9,3,5);  // Sub-matrix from rows 5, 6, 7, 8, 9 the columns 3, 4, and 5  
  a.get_row(10);  // Row 10
  a.get_col(10); // Column 10
  \endcode

  It is also possible to modify parts of a vector as e.g. in
  \code
  a.set_row(5, invector);    // Set row 5 to \c invector
  a.set_col(3, invector); // Set column 3 to \c invector
  a.copy_col(1, 5); // Copy column 5 to column 1
  a.swap_cols(1, 5); // Swap the contents of columns 1 and 5
  \endcode

  It is of course also possible to perform the common linear algebra
  methods such as addition, subtraction, and matrix multiplication. Observe
  though, that vectors are assumed to be column-vectors in operations with
  matrices.

  Most elementary functions such as sin(), cosh(), log(), abs(), ..., are
  available as operations on the individual elements of the matrices. Please
  see the individual functions for more details.
*/
template<class T>
class Mat {
 public:
  //! Class constructor
  Mat() { init(); }
  //! Create a matrix of size (inrow, incol)
  Mat(int inrow, int incol) {
    init(); it_assert1(inrow>=0 && incol>=0, "The rows and columns must be >= 0");
    alloc(inrow, incol); }
  //! Create a copy of the matrix \c m
  Mat(const Mat<T> &m);
  //! Create a copy of the vector \c invector treated as a column vector
  Mat(const Vec<T> &invector);
  //! Set matrix equal to values in string
  Mat(const std::string &str) { init(); set(str); }
  //! Set matrix equal to values in string
  Mat(const char *str) { init(); set(str); }

  /*! 
    \brief Constructor taking a C-array as input. Copies all data.
    
    By default the matrix is stored as a RowMajor matrix (i.e. listing elements in sequence
    beginning with the first column).
  */
  Mat(T *c_array, int rows, int cols, bool RowMajor = true);

  //! Class destructor
  ~Mat() { free(); }
    
  //! The number of columns
  int cols() const { return no_cols; }
  //! The number of rows
  int rows() const { return no_rows; }
  //! Set size of matrix. If copy = true then keep the data before resizing.
  void set_size(int inrow, int incol, bool copy=false);
  //! Set matrix equal to the all zero matrix
  void zeros();
  //! Set matrix equal to the all zero matrix
  void clear() { zeros(); }
  //! Set matrix equal to the all one matrix
  void ones();
  //! Set matrix equal to values in the string
  bool set(const char *str);
  //! Set matrix equal to values in the string
  bool set(const std::string &str);

  //! Get element (R,C) from matrix
  const T &operator()(int R,int C) const
    { it_assert0(R>=0 && R<no_rows && C>=0 && C<no_cols, 
		 "Mat<T>::operator(): index out of range"); return data[R+C*no_rows]; }
  //! Get element (R,C) from matrix
  T &operator()(int R,int C)
    { it_assert0(R>=0 && R<no_rows && C>=0 && C<no_cols, 
		 "Mat<T>::operator(): index out of range"); return data[R+C*no_rows]; }
  //! Get element \c index using linear addressing (by rows)
  T &operator()(int index)
    { it_assert0(index<no_rows*no_cols && index>=0,"Mat<T>::operator(): index out of range");
    return data[index]; }
  //! Get element \c index using linear addressing (by rows)
  const T &operator()(int index) const
    { it_assert0(index<no_rows*no_cols && index>=0,"Mat<T>::operator(): index out of range");
    return data[index]; }

  /*! 
    \brief Sub-matrix from row \c r1 to row \c r2 and columns \c c1 to \c c2.
    
    Value -1 indicates the last row and column, respectively.
  */
  const Mat<T> operator()(int r1, int r2, int c1, int c2) const;

  //! Get row \c Index  
  Vec<T> get_row(int Index) const ;
  //! Get rows \c r1 through \c r2
  Mat<T> get_rows(int r1, int r2) const;
  //! Get the rows specified by \c indexlist
  Mat<T> get_rows(const Vec<int> &indexlist) const;
  //! Get column \c Index
  Vec<T> get_col(int Index) const ;
  //! Get columns \c c1 through \c c2
  Mat<T> get_cols(int c1, int c2) const;
  //! Get the columns specified by \c indexlist
  Mat<T> get_cols(const Vec<int> &indexlist) const;
  //! Set row \c Index to \c invector
  void set_row(int Index, const Vec<T> &invector);
  //! Set column \c Index to \c invector
  void set_col(int Index, const Vec<T> &invector);
  //! Copy row \c from onto row \c to
  void copy_row(int to, int from);
  //! Copy column \c from onto column \c to
  void copy_col(int to, int from);
  //! Swap the rows \c r1 and \c r2
  void swap_rows(int r1, int r2);
  //! Swap the columns \c c1 and \c c2
  void swap_cols(int c1, int c2);

  //! Set submatrix defined by rows r1,r2 and columns c1,c2 to matrix m
  void set_submatrix(int r1, int r2, int c1, int c2, const Mat<T> &m);
  //! Set all elements of submatrix defined by rows r1,r2 and columns c1,c2 to value t
  void set_submatrix(int r1, int r2, int c1, int c2, const T t);

  //! Concatenate the matrices \c m1 and \c m2 horizontally
  friend Mat<T> concat_horizontal TEMPLATE_FUN(const Mat<T> &m1, const Mat<T> &m2);
  //! Concatenate the matrices \c m1 and \c m2 vertically
  friend Mat<T> concat_vertical TEMPLATE_FUN(const Mat<T> &m1, const Mat<T> &m2);

  //! Set all elements of the matrix equal to \c t
  void operator=(T t);
  //! Set matrix equal to \c m
  void operator=(const Mat<T> &m);
  //! Set matrix equal to the vector \c v, assuming column vector
  void operator=(const Vec<T> &v);
  //! Set matrix equal to values in the string
  void operator=(const char *values) { set(values); }

  //! Addition of matrices
  void operator+=(const Mat<T> &m);
  //! Addition of scalar to matrix
  void operator+=(T t);
  //! Addition of two matrices
  friend Mat<T> operator+TEMPLATE_FUN(const Mat<T> &m1, const Mat<T> &m2);
  //! Addition of matrix and scalar
  friend Mat<T> operator+TEMPLATE_FUN(const Mat<T> &m, T t);
  //! Addition of scalar and matrix
  friend Mat<T> operator+TEMPLATE_FUN(T t, const Mat<T> &m);

  //! Subtraction of matrix
  void operator-=(const Mat<T> &m);
  //! Subtraction of scalar from matrix
  void operator-=(T t);
  //! Subtraction of \c m2 from \c m1
  friend Mat<T> operator-TEMPLATE_FUN(const Mat<T> &m1, const Mat<T> &m2);
  //! Subraction of scalar from matrix
  friend Mat<T> operator-TEMPLATE_FUN(const Mat<T> &m, T t);
  //! Subtract matrix from scalar
  friend Mat<T> operator-TEMPLATE_FUN(T t, const Mat<T> &m);
  //! Subraction of matrix 
  friend Mat<T> operator-TEMPLATE_FUN(const Mat<T> &m);

  //! Matrix multiplication
  void operator*=(const Mat<T> &m);
  //! Multiplication by a scalar
  void operator*=(T t);
  //! Multiplication of two matrices
  friend Mat<T> operator*TEMPLATE_FUN(const Mat<T> &m1, const Mat<T> &m2);
  //! Multiplication of matrix \c m and vector \c v (column vector)
  friend Vec<T> operator*TEMPLATE_FUN(const Mat<T> &m, const Vec<T> &v);
  //! Multiplication of transposed vector \c v and matrix \c m
  friend Vec<T> operator*TEMPLATE_FUN(const Vec<T> &v, const Mat<T> &m);
  //! Multiplication of matrix and scalar
  friend Mat<T> operator*TEMPLATE_FUN(const Mat<T> &m, T t);
  //! Multiplication of scalar and matrix
  friend Mat<T> operator*TEMPLATE_FUN(T t, const Mat<T> &m);
  //! Elementwise multiplication of two matrices
  friend Mat<T> elem_mult TEMPLATE_FUN(const Mat<T> &m1, const Mat<T> &m2);

  //! Division by a scalar
  void operator/=(T t);
  //! Division of matrix with scalar
  friend Mat<T> operator/TEMPLATE_FUN(const Mat<T> &m, T t);
  //! Elementwise division with the current matrix
  void operator/=(const Mat<T> &m);
  //! Elementwise division of matrix \c m1 with matrix \c m2
  friend Mat<T> elem_div TEMPLATE_FUN(const Mat<T> &m1, const Mat<T> &m2);

  //! Compare two matrices. False if wrong sizes or different values
  bool operator==(const Mat<T> &m) const;
  //! Compare two matrices. True if different
  bool operator!=(const Mat<T> &m) const;

  //! Get element (R,C) from matrix without boundary check (Not recommended to use). 
  T &_elem(int R,int C) {  return data[R+C*no_rows]; }
  //! Get element (R,C) from matrix without boundary check (Not recommended to use).
  const T &_elem(int R,int C) const {  return data[R+C*no_rows]; }
  //! Get element \c index using linear addressing (by rows) without boundary check (Not recommended to use).
  T &_elem(int index) { return data[index]; }
  //! Get element \c index using linear addressing (by rows) without boundary check (Not recommended to use).
  const T &_elem(int index) const { return data[index]; }

  //! Access of the internal data structure. Don't use. May be changed!
  T *_data() { return data; }
  //! Access to the internal data structure. Don't use. May be changed!
  const T *_data() const { return data; }
  //! Access to the internal data structure. Don't use. May be changed!
  int _datasize() const { return datasize; }

 protected:
  //! Allocate memory for the matrix
  void alloc(int rows, int cols)
    {
      if ( datasize == rows * cols ) { // Reuse the memory
	no_rows = rows; no_cols = cols;
	return;
      }
      free();  // Free memory (if any allocated)
      if (rows == 0 || cols == 0)
	return;
      
      datasize = rows * cols;
      data = new T[datasize];
      no_rows = rows; no_cols = cols;
      
      it_assert1(data, "Mat<T>::alloc: Out of memory");
    }

  //! Free the memory space of the matrix
  void free() { delete [] data; init(); }

  //! Protected integer variables
  int datasize, no_rows, no_cols;

  //! Protected data pointer
  T *data;

 private:
  void init()
    {
      data = 0;
      datasize = no_rows = no_cols = 0;
    }
};



/*! 
  \relates Mat
  \brief Output stream for matrices
*/
template <class T>
std::ostream &operator<<(std::ostream &os, const Mat<T> &m);

/*! 
  \relates Mat
  \brief Input stream for matrices
*/
template <class T>
std::istream &operator>>(std::istream &is, Mat<T> &m);

/*! 
  \relates Mat
  \brief Default Matrix Type
*/
typedef Mat<double> mat;

/*! 
  \relates Mat
  \brief Default Complex Matrix Type
*/
typedef Mat<double_complex> cmat;

/*! 
  \relates Mat
  \brief Integer matrix
*/
typedef Mat<int> imat;

/*! 
  \relates Mat
  \brief long_long matrix
*/
typedef Mat<long_long> llmat;

/*! 
  \relates Mat
  \brief short int matrix
*/
typedef Mat<short int> smat;

/*! 
  \relates Mat
  \brief bin matrix
*/
typedef Mat<bin> bmat;


//------------------ Inline definitions ----------------------------

template<class T> inline
Mat<T>::Mat<T>(T *c_array, int rows, int cols, bool RowMajor)
{
  init();
  alloc(rows, cols);
  
  if (!RowMajor)
    memcpy(data, c_array, datasize*sizeof(T));
  else { // Row Major
    T *ptr = c_array;
    for (int i=0; i<rows; i++) {
      for (int j=0; j<cols; j++)
	data[i+j*no_rows] = *(ptr++);
    }
  }
}


template<class T> inline
const Mat<T> Mat<T>::operator()(int r1, int r2, int c1, int c2) const
{
  if (r1 == -1) r1 = no_rows-1;
  if (r2 == -1) r2 = no_rows-1;
  if (c1 == -1) c1 = no_cols-1;
  if (c2 == -1) c2 = no_cols-1;

  it_assert1(r1>=0 && r2>=0 && r1<no_rows && r2<no_rows &&
	     c1>=0 && c2>=0 && c1<no_cols && c2<no_cols, "operator()(r1,r2,c1,c2)");

  it_assert1(r2>=r1 && c2>=c1, "Mat<T>::op(): r2>=r1 or c2>=c1 not fulfilled");

  Mat<T> s(r2-r1+1, c2-c1+1);

  for (int i=0;i<s.no_cols;i++)
    memcpy(s.data+i*s.no_rows, data+r1+(c1+i)*no_rows, s.no_rows*sizeof(T));

  return s;
}

#endif // __mat_h
 
