/*---------------------------------------------------------------------------*
 *                                   IT++			             *
 *---------------------------------------------------------------------------*
 * Copyright (c) 1995-2001 by Tony Ottosson, Thomas Eriksson, Pl Frenger,   *
 * Tobias Ringstrm, and Jonas Samuelsson.                                   *
 *                                                                           *
 * Permission to use, copy, modify, and distribute this software and its     *
 * documentation under the terms of the GNU General Public License is hereby *
 * granted. No representations are made about the suitability of this        *
 * software for any purpose. It is provided "as is" without expressed or     *
 * implied warranty. See the GNU General Public License for more details.    *
 *---------------------------------------------------------------------------*/

/*! 
  \file 
  \brief Definition of an argument parser class
  \author Thomas Eriksson and Pl Frenger (Thanks to Svante signell for valuable input)

  1.9

  2003/01/13 20:47:50
*/

#ifndef __parser_h
#define __parser_h

#define MAX_STR_LEN 4096

#include "base/vec.h"
#include "base/mat.h"
#include "base/array.h"
#include <fstream>
#include <string>

using namespace std;

/*! 
  \brief Argument Parser Class 
  \author Thomas Eriksson and Pl Frenger (Thanks to Svante Signell for valuable input)

  This class parses strings to variables. The syntax is compatible with Matlab and Octave.
  It can be used in several different ways. The following test program
  and test data file gives 5 examples:

  <ul>
  <li> Use The Parser class on a parameter file </li>
  <li> Use the Parser class on the command line input </li>
  <li> Use the Parser class on a char pointer (usually the command line input) </li>
  <li> Use the Parser class on a parameter file and a char pointer </li>
  <li> Use the Parser class on an Array of strings </li>
  </ul>
 
  The test program looks as follows:
  \include parser_test.cpp

  The data file \c parser_test_data.txt looks as follows:
  \include parser_test_data.txt
 */
class Parser {
public:

  //! Default Constructor
  Parser();

  //! Constructor. Sets input file name.
  Parser(const string &filename);

  //! Constructor. Uses argc and argv (command line arguments)
  Parser(int argc, char *argv[]);

  //! Constructor. Sets input file name and uses argc and argv (command line arguments)
  Parser(const string &filename, int argc, char *argv[]);

  //! Constructor. Sets and Array of strings
  Parser(const Array<string> &setup);

  //! Initialization function. Sets input file name.
  void init(const string &filename);

  //! Initialization function. Uses argc and argv (command line arguments)
  void init(int argc, char *argv[]);

  //! Initialization function. Sets input file name and uses argc and argv (command line arguments)
  void init(const string &filename, int argc, char *argv[]);

  //! Initialization function. Sets and Array of strings
  void init(const Array<string> &setup);

  //! Interpret variable \a name as a integer
  int get_int(const string &name,  int num=-1);

  //! Interpret variable \a name as a double
  double get_double(const string &name, int num=-1);

  //! Interpret variable \a name as a string
  string get_string(const string &name, int num=-1);

  //! Interpret variable \a name as a vec
  vec get_vec(const string &name, int num=-1);

  //! Interpret variable \a name as a ivec
  ivec get_ivec(const string &name, int num=-1);

  //! Interpret variable \a name as a svec
  svec get_svec(const string &name, int num=-1);

  //! Interpret variable \a name as a bvec
  bvec get_bvec(const string &name, int num=-1);

  //! Interpret variable \a name as a mat
  mat get_mat(const string &name, int num=-1);

  //! Interpret variable \a name as a imat
  imat get_imat(const string &name, int num=-1);

  //! Interpret variable \a name as a smat
  smat get_smat(const string &name, int num=-1);

  //! Interpret variable \a name as a bmat
  bmat get_bmat(const string &name, int num=-1);
 
protected:
private:

  //! Find the string \c name
  string findname(const string &name, bool &print_flag, int num=0);

  void pre_parsing(void);
  Array<string> SetupStrings;

};

#endif
