/*---------------------------------------------------------------------------*
 *                                   IT++			             *
 *---------------------------------------------------------------------------*
 * Copyright (c) 1995-2001 by Tony Ottosson, Thomas Eriksson, Pl Frenger,   *
 * Tobias Ringstrm, and Jonas Samuelsson.                                   *
 *                                                                           *
 * Permission to use, copy, modify, and distribute this software and its     *
 * documentation under the terms of the GNU General Public License is hereby *
 * granted. No representations are made about the suitability of this        *
 * software for any purpose. It is provided "as is" without expressed or     *
 * implied warranty. See the GNU General Public License for more details.    *
 *---------------------------------------------------------------------------*/

/*! 
  \file 
  \brief Sparse Matrix Class Definitions
  \author Tony Ottosson and Tobias Ringstrm

  1.7

  2002/12/19 23:56:44
*/

#ifndef __smat_h
#define __smat_h

#include "itconfig.h"

template <class T> class Vec;
template <class T> class Mat;
template <class T> class Sparse_Vec;
template <class T> class Sparse_Mat;

// ------------------------ Sparse_Mat Friends -------------------------------------

//! m1+m2 where m1 and m2 are sparse matrices  
template <class T>
Sparse_Mat<T> operator+(const Sparse_Mat<T> &m1, const Sparse_Mat<T> &m2);

//! m1*m2 where m1 and m2 are sparse matrices
template <class T>
Sparse_Mat<T> operator*(const Sparse_Mat<T> &m1, const Sparse_Mat<T> &m2);

//! m*v where m is a sparse matrix and v is a full column vector
template <class T>
Vec<T> operator*(const Sparse_Mat<T> &m, const Vec<T> &v);

//! v'*m where m is a sparse matrix and v is a full column vector
template <class T>
Vec<T> operator*(const Vec<T> &v, const Sparse_Mat<T> &m);

//! m'*m where m is a sparse matrix
template <class T>
Mat<T> trans_mult(const Sparse_Mat<T> &m);

//! m'*m where m is a sparse matrix
template <class T>
Sparse_Mat<T> trans_mult_s(const Sparse_Mat<T> &m);

//! m1'*m2 where m1 and m2 are sparse matrices
template <class T>
Sparse_Mat<T> trans_mult(const Sparse_Mat<T> &m1, const Sparse_Mat<T> &m2);

//! m'*v where m is a sparse matrix and v is a full column vector
template <class T>
Vec<T> trans_mult(const Sparse_Mat<T> &m, const Vec<T> &v);

//! m1*m2' where m1 and m2 are sparse matrices
template <class T>
Sparse_Mat<T> mult_trans(const Sparse_Mat<T> &m1, const Sparse_Mat<T> &m2);

/*! 
  \brief Templated Sparse Matrix Class
  \author Tony Ottosson and Tobias Ringstrom
*/
template <class T>
class Sparse_Mat {
public:

  //! Default constructor
  Sparse_Mat();

  /*! 
    \brief Initiate an empty sparse vector

    A Sparse_Mat consists of colums that have the type Sparse_Vec. The maximum number of non-zero elements is each column
    is denoted \c row_data_init.

    \param rows Number of rows in the matrix
    \param cols Number of columns in the matrix
    \param row_data_init The maximum number of non-zero elements in each column (default value is 200)
  */
  Sparse_Mat(int rows, int cols, int row_data_init=200);

  //! Initiate a new sparse matrix. The elements of \c m are copied into the new sparse matrix
  Sparse_Mat(const Sparse_Mat<T> &m);

  //! Initiate a new sparse matrix from a dense matrix. The elements of \c m are copied into the new sparse matrix
  Sparse_Mat(const Mat<T> &m);

  /*! 
    \brief Initiate a new sparse matrix from a dense matrix. Elements of \c m larger than \c epsilon are copied into the new sparse matrix.
    
    \note If the type T is double complex, then the elements of \c m larger than \c abs(epsilon) are copied into the new sparse matrix.
  */
  Sparse_Mat(const Mat<T> &m, T epsilon);

  //! Destructor
  ~Sparse_Mat();
  
  /*! 
    \brief Set the size of the sparse matrix

    A Sparse_Mat consists of colums that have the type Sparse_Vec. The maximum number of non-zero elements is each column
    is denoted \c row_data_init.

    \param rows Number of rows in the matrix
    \param cols Number of columns in the matrix
    \param row_data_init The maximum number of non-zero elements in each column (default value is 200)
  */
  void set_size(int rows, int cols, int row_data_init=200);
  
  //! Returns the number of rows of the sparse matrix
  int rows() const { return n_rows; }

  //! Returns the number of columns of the sparse matrix
  int cols() const { return n_cols; }

  //! The number of non-zero elements in the sparse matrix
  int nnz() const;

  //! Returns the density of the sparse matrix: (number of non-zero elements)/(total number of elements)
  double density() const;
  
  //! Set the maximum number of non-zero elements in each column equal to the actual number of non-zero elements in each column
  void compact();
  
  //! Returns a full, dense matrix in \c m
  void full(Mat<T> &m) const;

  //! Returns a full, dense matrix 
  Mat<T> full() const;
  
  //! Returns element of row \c r and column \c c
  T operator()(int r, int c) const;

  //! Set element (\c r, \c c ) equal to \c v
  void set(int r, int c, T v);

  //! Set a new element with index (\c r, \c c ) equal to \c v
  void set_new(int r, int c, T v);

  //! Returns the sub-matrix from rows \c r1 to \c r2 and columns \c c1 to \c c2
  Sparse_Mat<T> get_submatrix(int r1, int r2, int c1, int c2) const;

  //! Returns the sub-matrix from columns \c c1 to \c c2 (all rows)
  Sparse_Mat<T> get_submatrix_cols(int c1, int c2) const;

  //! Returns column \c c of the Sparse_Mat in the Sparse_Vec \c v
  void get_col(int c, Sparse_Vec<T> &v);

  //! Returns column \c c of the Sparse_Mat 
  Sparse_Vec<T> get_col(int c);

  //! Set column \c c of the Sparse_Mat 
  void set_col(int c, const Sparse_Vec<T> &v);
  
  //! Transpose the sparse matrix
  void transpose(Sparse_Mat<T> &m) const;

  //! Returns the transpose of the sparse matrix
  Sparse_Mat<T> transpose() const;
  
  //! Assign sparse matrix the value and dimensions of the sparse matrix \c m
  void operator=(const Sparse_Mat<T> &m);

  //! Assign sparse matrix the value and dimensions of the dense matrix \c m
  void operator=(const Mat<T> &m);
  
  //! Returns the sign inverse of all elements in the sparse matrix
  Sparse_Mat<T> operator-() const;
  
  //! Compare two sparse matricies. False if wrong sizes or different values
  bool operator==(const Sparse_Mat<T> &m) const;
  
  //! Addition m1+m2 where m1 and m2 are sparse matrices
  friend Sparse_Mat<T> operator+TEMPLATE_FUN(const Sparse_Mat<T> &m1, const Sparse_Mat<T> &m2);

  //! Multiplication m1*m2 where m1 and m2 are sparse matrices
  friend Sparse_Mat<T> operator*TEMPLATE_FUN(const Sparse_Mat<T> &m1, const Sparse_Mat<T> &m2);

  //! Multiplication m*v where m is a sparse matrix and v is a full column vector
  friend Vec<T> operator*TEMPLATE_FUN(const Sparse_Mat<T> &m, const Vec<T> &v);

  //! Multiplication v'*m where m is a sparse matrix and v is a full column vector
  friend Vec<T> operator*TEMPLATE_FUN(const Vec<T> &v, const Sparse_Mat<T> &m);

  //! Multiplication m'*m where m is a sparse matrix. Returns a full, dense matrix
  friend Mat<T> trans_mult TEMPLATE_FUN(const Sparse_Mat<T> &m);

  //! Multiplication m'*m where m is a sparse matrix, Returns a sparse matrix
  friend Sparse_Mat<T> trans_mult_s TEMPLATE_FUN(const Sparse_Mat<T> &m);

  //! Multiplication m1'*m2 where m1 and m2 are sparse matrices
  friend Sparse_Mat<T> trans_mult TEMPLATE_FUN(const Sparse_Mat<T> &m1, const Sparse_Mat<T> &m2);

  //! Multiplication m'*v where m is a sparse matrix and v is a full column vector
  friend Vec<T> trans_mult TEMPLATE_FUN(const Sparse_Mat<T> &m, const Vec<T> &v);

  //! Multiplication m1*m2' where m1 and m2 are sparse matrices
  friend Sparse_Mat<T> mult_trans TEMPLATE_FUN(const Sparse_Mat<T> &m1, const Sparse_Mat<T> &m2);
  
 private:
  void init();
  void alloc_empty();
  void alloc(int row_data_size=200);
  void free();
  
  int n_rows, n_cols;
  Sparse_Vec<T> *col;
};

/*! 
  \relates Sparse_Mat
  \brief Sparse integer matrix
*/
typedef Sparse_Mat<int> sparse_imat;

/*! 
  \relates Sparse_Mat
  \brief Sparse double matrix
*/
typedef Sparse_Mat<double> sparse_mat;

/*! 
  \relates Sparse_Mat
  \brief Sparse double_complex matrix
*/
typedef Sparse_Mat<double_complex> sparse_cmat;

//---------------------- Inline functions --------------------------------

#ifndef DOXYGEN_SHOULD_SKIP_THIS

template <class T>
inline Sparse_Mat<T> sparse(const Mat<T> &m, T epsilon)
{
    Sparse_Mat<T> s(m, epsilon);
    return s;
}

template <class T>
inline Mat<T> full(const Sparse_Mat<T> &s)
{
    Mat<T> m;
    s.full(m);
    return m;
}

template <class T>
inline Sparse_Mat<T> transpose(const Sparse_Mat<T> &s)
{
    Sparse_Mat<T> m;
    s.transpose(m);
    return m;
}

// template <class T>
// std::ostream &operator<<(std::ostream &os, const Sparse_Mat<T> &m);

// template <class T>
// std::istream &operator>>(std::istream &is, Sparse_Mat<T> &m);

#endif //DOXYGEN_SHOULD_SKIP_THIS

#endif //__smat_h
