/*---------------------------------------------------------------------------*
 *                                   IT++			             *
 *---------------------------------------------------------------------------*
 * Copyright (c) 1995-2001 by Tony Ottosson, Thomas Eriksson, Pl Frenger,   *
 * Tobias Ringstrm, and Jonas Samuelsson.                                   *
 *                                                                           *
 * Permission to use, copy, modify, and distribute this software and its     *
 * documentation under the terms of the GNU General Public License is hereby *
 * granted. No representations are made about the suitability of this        *
 * software for any purpose. It is provided "as is" without expressed or     *
 * implied warranty. See the GNU General Public License for more details.    *
 *---------------------------------------------------------------------------*/

/*! 
  \file 
  \brief Sparse Vector Class Definitions 
  \author Tony Ottosson, and Tobias Ringstrm

  1.8

  2002/12/19 23:56:44
*/

#ifndef __itpp_svec_h
#define __itpp_svec_h

#include "itconfig.h"

template <class T> class Vec;
template <class T> class Sparse_Vec;

// ----------------------- Sparse_Vec Friends -------------------------------

//! v1+v2 where v1 and v2 are sparse vector
template <class T>
Sparse_Vec<T> operator+(const Sparse_Vec<T> &v1, const Sparse_Vec<T> &v2);

//! v1*v2 where v1 and v2 are sparse vectors
template <class T>
T operator*(const Sparse_Vec<T> &v1, const Sparse_Vec<T> &v2);

//! v1*v2 where v1 is a sparse vector and v2 is a dense vector
template <class T>
T operator*(const Sparse_Vec<T> &v1, const Vec<T> &v2);

//! v1*v2 where v1 is a dense vector and v2 is a sparse vector
template <class T>
T operator*(const Vec<T> &v1, const Sparse_Vec<T> &v2);

//! Elementwise multiplication of two sparse vectors returning a sparse vector
template <class T>
Sparse_Vec<T> elem_mult(const Sparse_Vec<T> &v1, const Sparse_Vec<T> &v2);

//! Elementwise multiplication of a sparse vector and a dense vector returning a dense vector
template <class T>
Vec<T> elem_mult(const Sparse_Vec<T> &v1, const Vec<T> &v2);

//! Elementwise multiplication of a sparse vector and a dense vector returning a sparse vector
template <class T>
Sparse_Vec<T> elem_mult_s(const Sparse_Vec<T> &v1, const Vec<T> &v2);

//! Elementwise multiplication of a dense vector and a sparse vector returning a dense vector
template <class T>
Vec<T> elem_mult(const Vec<T> &v1, const Sparse_Vec<T> &v2);

//! Elementwise multiplication of a dense vector and a sparse vector returning a sparse vector
template <class T>
Sparse_Vec<T> elem_mult_s(const Vec<T> &v1, const Sparse_Vec<T> &v2);

/*! 
  \brief Templated sparse vector class
  \author Tony Ottosson and Tobias Ringstrom
*/
template <class T>
class Sparse_Vec {
public:

  //! Default constructor
  Sparse_Vec();

  /*!
    \brief Initiate an empty sparse vector

    \param sz Size of the sparse vector (i.e. maximum index is (\c sz - 1))
    \param data_init Maximum number of non-zero elements in the sparse vector (default value 200)
  */
  Sparse_Vec(int sz, int data_init=200);

  //! Initiate a new sparse vector. The elements of \c v are copied into the new sparse vector
  Sparse_Vec(const Sparse_Vec<T> &v);

  //! Initiate a new sparse vector from a dense vector. The elements of \c v are copied into the new sparse vector
  Sparse_Vec(const Vec<T> &v);

  /*! 
    \brief Initiate a new sparse vector from a dense vector. Elements of \c v larger than \c epsilon are copied into the new sparse vector.
    
    \note If the type T is double complex, then the elements of \c v larger than \c abs(epsilon) are copied into the new sparse vector.
  */
  Sparse_Vec(const Vec<T> &v, T epsilon);

  //! Destructor
  ~Sparse_Vec();
  
  /*! 
    \brief Set the size of the sparse vector

    \param sz Size of the sparse vector (i.e. maximum index is (\c sz - 1))
    \param data_init Maximum number of non-zero elements in the sparse vector (default value 200)
  */
  void set_size(int sz, int data_init=200);
  
  //! Returns the size of the sparse vector
  int size() const { return v_size; }

  //! Number of non-zero elements in the sparse vector
  int nnz() const { return used_size; }

  //! Returns the density of the sparse vector: (number of non-zero elements)/(size of the vector)
  double density() const;
  
  //! Set the maximum number of non-zero elements to \c new_size
  void resize_data(int new_size);

  //! Set the maximum number of non-zero elements equal to the actual number of non-zero elements
  void compact();
  
  //! Returns a full, dense vector in \c v
  void full(Vec<T> &v) const;

  //! Returns a full, dense vector
  Vec<T> full() const;
  
  //! Returns the element with index \c i
  T operator()(int i) const;

  //! Set element \c i equal to \c v
  void set(int i, T v);

  //! Set a new element with index \c i equal to \c v
  void set_new(int i, T v);

  //! ACTION: ADD DOCUMENTATION FOR THIS MEMBER
  T &get_nz_data(int p) { return data[p]; }

  //! Returns the p-th non-zero data element
  T get_nz_data(int p) const { return data[p]; }

  //! ACTION: ADD DOCUMENTATION FOR THIS MEMBER
  int &get_nz_index(int p) { return index[p]; }

  //! Returns the vector index of the p-th non-zero element
  int get_nz_index(int p) const { return index[p]; }

  //! Returns the p-th non-zero value in \c dat and the corresponding vector index in \c idx
  void get_nz(int p, int &idx, T &dat) const { idx=index[p]; dat=data[p]; }

  //! Return sparse subvector from index \c i1 to index \c i2
  Sparse_Vec<T> get_subvector(int i1, int i2) const;
  
  //! Returns the sum of all values squared
  T sqr() const;
  
  //! Assign sparse vector the value and length of the sparse vector \c v
  void operator=(const Sparse_Vec<T> &v);
  //! Assign sparse vector the value and length of the dense vector \c v
  void operator=(const Vec<T> &v);
  
  //! Returns the sign inverse of all elements in the sparse vector
  Sparse_Vec<T> operator-() const;

  //! Compare two sparse vectors. False if wrong sizes or different values
  bool operator==(const Sparse_Vec<T> &v) const;
  
  //! Addition v1+v2 where v1 and v2 are sparse vector
  friend Sparse_Vec<T> operator+TEMPLATE_FUN(const Sparse_Vec<T> &v1, const Sparse_Vec<T> &v2);
  //! Scalar product v1*v2 where v1 and v2 are sparse vectors
  friend T operator*TEMPLATE_FUN(const Sparse_Vec<T> &v1, const Sparse_Vec<T> &v2);
  //! Scalar product v1*v2 where v1 is a sparse vector and v2 is a dense vector
  friend T operator*TEMPLATE_FUN(const Sparse_Vec<T> &v1, const Vec<T> &v2);
  //! Scalar product v1*v2 where v1 is a dense vector and v2 is a sparse vector
  friend T operator*TEMPLATE_FUN(const Vec<T> &v1, const Sparse_Vec<T> &v2);

  //! Element wise multiplication of two sparse vectors
  friend Sparse_Vec<T> elem_mult TEMPLATE_FUN(const Sparse_Vec<T> &v1, const Sparse_Vec<T> &v2);

  //! Element wise multiplication of a sparse vector and a dense vector
  friend Vec<T> elem_mult TEMPLATE_FUN(const Sparse_Vec<T> &v1, const Vec<T> &v2);

  //! Element wise multiplication of a sparse vector and a dense vector returning a sparse vector
  friend Sparse_Vec<T> elem_mult_s TEMPLATE_FUN(const Sparse_Vec<T> &v1, const Vec<T> &v2);

  //! Element wise multiplication of a a dense vector and a sparse vector
  friend Vec<T> elem_mult TEMPLATE_FUN(const Vec<T> &v1, const Sparse_Vec<T> &v2);

  //! Element wise multiplication of a a dense vector and a sparse vector returning a sparse vector
  friend Sparse_Vec<T> elem_mult_s TEMPLATE_FUN(const Vec<T> &v1, const Sparse_Vec<T> &v2);
  
 private:
  void init();
  void alloc();
  void free();
  
  int v_size, used_size, data_size;
  T *data;
  int *index;
};

/*! 
  \relates Sparse_Vec
  \brief Definition of an integer sparce vector
*/
typedef Sparse_Vec<int> sparse_ivec;

/*! 
  \relates Sparse_Vec
  \brief Definition of a double sparce vector
*/
typedef Sparse_Vec<double> sparse_vec;

/*! 
  \relates Sparse_Vec
  \brief Definition of a double_complex sparce vector
*/
typedef Sparse_Vec<double_complex> sparse_cvec;

// ----------------------- Inline functions --------------------------------

#ifndef DOXYGEN_SHOULD_SKIP_THIS

template <class T>
inline Sparse_Vec<T> sparse(const Vec<T> &v)
{
    Sparse_Vec<T> s(v);
    return s;
}

template <class T>
inline Sparse_Vec<T> sparse(const Vec<T> &v, T epsilon)
{
    Sparse_Vec<T> s(v, epsilon);
    return s;
}

template <class T>
inline Vec<T> full(const Sparse_Vec<T> &s)
{
    Vec<T> v;
    s.full(v);
    return v;
}

// template <class T>
// std::ostream &operator<<(std::ostream &os, const Sparse_Vec<T> &v);

// template <class T>
// std::istream &operator>>(std::istream &is, Sparse_Vec<T> &v);

#endif //DOXYGEN_SHOULD_SKIP_THIS

#endif //__itpp_svec_h
