/*---------------------------------------------------------------------------*
 *                                   IT++			             *
 *---------------------------------------------------------------------------*
 * Copyright (c) 1995-2002 by Tony Ottosson, Thomas Eriksson, Pl Frenger,   *
 * Tobias Ringstrm, and Jonas Samuelsson.                                   *
 *                                                                           *
 * Permission to use, copy, modify, and distribute this software and its     *
 * documentation under the terms of the GNU General Public License is hereby *
 * granted. No representations are made about the suitability of this        *
 * software for any purpose. It is provided "as is" without expressed or     *
 * implied warranty. See the GNU General Public License for more details.    *
 *---------------------------------------------------------------------------*/

/*! 
  \file 
  \brief Templated Vector Class Definitions
  \author Tony Ottosson and Tobias Ringstrm

  1.16

  2003/01/04 00:21:54
*/

#ifndef __vec_h
#define __vec_h

// This needs to be included before algorithm included in scalfunc for g++-2.7.2
#include <string> // Not the same as string.h!
#include "base/binary.h"
#include "itconfig.h"
#include "base/itassert.h"
#include <iostream>
#include <complex>
#include <cstdlib>
#include <cstring>

//! Default vector class of type \c double. Usually double
template<class T> class Vec;
//! Default matrix class of type \c double. Usually double
template<class T> class Mat;
class bin;

//----------------- Vec Friends -------------------------
//! Addition of two vectors
template<class T> Vec<T> operator+(const Vec<T> &v1, const Vec<T> &v2);
//! Addition of a vector and a scalar
template<class T> Vec<T> operator+(const Vec<T> &v, T t);
//! Addition of a scalar and a vector
template<class T> Vec<T> operator+(T t, const Vec<T> &v);
//! Subtraction of a vector from a vector
template<class T> Vec<T> operator-(const Vec<T> &v1, const Vec<T> &v2);
//! Subtraction of a scalar from a vector
template<class T> Vec<T> operator-(const Vec<T> &v, T t);
//! Subtraction of vector from scalar. Results in a vector
template<class T> Vec<T> operator-(T t, const Vec<T> &v);
//! Negation of vector
template<class T> Vec<T> operator-(const Vec<T> &v);
//! Inner (dot) product of two vectors v1 and v2
template<class T> T dot(const Vec<T> &v1, const Vec<T> &v2);
//! Inner (dot) product of two vectors v1 and v2
template<class T> T operator*(const Vec<T> &v1, const Vec<T> &v2)
  { return dot(v1, v2); }
//! Outer product of two vectors v1 and v2
template<class T> Mat<T> outer_product(const Vec<T> &v1, const Vec<T> &v2);
//! Multiplication of a vector and a scalar
template<class T> Vec<T> operator*(const Vec<T> &v, T t);
//! Multiplication of a scalar and a vector. Results in a vector
template<class T> Vec<T> operator*(T t, const Vec<T> &v);
//! Elementwise multiplication of the two vectors
template<class T> Vec<T> elem_mult(const Vec<T> &v1, const Vec<T> &v2);
//! Elementwise multiplication of the three vectors
template<class T> Vec<T> elem_mult(const Vec<T> &v1, const Vec<T> &v2, const Vec<T> &v3);
//! Elementwise multiplication of the four vectors
template<class T> Vec<T> elem_mult(const Vec<T> &v1, const Vec<T> &v2, const Vec<T> &v3, const Vec<T> &v4);

//! Division of all elements in \c v with \c t
template<class T> Vec<T> operator/(const Vec<T> &v, T t);
//! Division of \c t with all elements in \c v
template<class T> Vec<T> operator/(const T t, const Vec<T> &v);
//! Elementwise division
template<class T> Vec<T> elem_div(const Vec<T> &v1, const Vec<T> &v2);
//! Elementwise division of scalar \c t and vector \c v
template<class T> Vec<T> elem_div(const T t, const Vec<T> &v);

//! Append element \c a to the end of the vector \c v
template<class T> Vec<T> concat(const Vec<T> &v, const T a);
//! Concat element \c a to the beginning of the vector \c v
template<class T> Vec<T> concat(const T a, const Vec<T> &v);
//! Concat vectors \c v1 and \c v2
template<class T> Vec<T> concat(const Vec<T> &v1,const Vec<T> &v2);
//! Concat vectors \c v1, \c v2 and \c v3
template<class T> Vec<T> concat(const Vec<T> &v1, const Vec<T> &v2, const Vec<T> &v3);
//! Concat vectors \c v1, \c v2, \c v3 and \c v4
template<class T> Vec<T> concat(const Vec<T> &v1, const Vec<T> &v2, const Vec<T> &v3, const Vec<T> &v4);
//! Concat vectors \c v1, \c v2 \c v3, \c v4 and \c v5
template<class T> Vec<T> concat(const Vec<T> &v1, const Vec<T> &v2, const Vec<T> &v3, 
				const Vec<T> &v4, const Vec<T> &v5);

/*! 
  \brief Templated vectors
  \author Tony Ottosson and Tobias Ringstrom

  Vectors can be of arbitrarily types, but conversions and functions are
  prepared for \c bin, \c short, \c int, \c double, and \c double_complex
  vectors and these are predefined as: \c bvec, \c svec, \c ivec, \c vec,
  and \c cvec. \c double and \c double_complex are \c double and
  \c complex<double> respectively.

  Examples:

  Vector Constructors:
  When constructing a vector without a length (memory) use
  \code vec temp; \endcode
  For construction of a vector of a given length use
  \code vec temp(length); \endcode
  It is also possible to assign the constructed vector the value and size
  of another vector by
  \code vec temp(invector); \endcode
  If you have explicit values you would like to assign to the vector it is
  possible to do this using strings as:
  \code
  vec a("0 0.7 5 9.3"); // that is a = [0, 0.7, 5, 9.3]
  vec a="0 0.7 5 9.3";  // the constructor are called implicitly
  ivec b="0:5";  // that is b = [0, 1, 2, 3, 4, 5]
  vec c="3:2.5:13";  // that is c = [3, 5.5, 8, 10.5, 13]
  \endcode
  It is also possible to change length by
  \code temp.set_size(new_length, false); \endcode
  where \c false is used to indicate that the old values in \c temp
  is not copied. If you like to preserve the values use \c true.

  There are a number of methods to access parts of a vector. Examples are
  \code
  a(5);     // Element number 5
  a(5,9);  // Elements 5, 6, 7, 8, and 9
  a.left(10);  // The 10 most left elements (the first)
  a.right(10); // The 10 most right elements (the last)
  a.mid(5, 7); // 7 elements starting from element 5
  \endcode

  It is also possible to modify parts of a vector as e.g. in
  \code
  a.del(5);    // deletes element number 5
  a.ins(3.4, 9); // inserts the element 3.4 at position 9
  a.replace_mid(12, b); // replaces elements from 12 with the vector b
  \endcode

  It is of course also possible to perform the common linear algebra
  methods such as addition, subtraction, and scalar product (*). Observe
  though, that vectors are assumed to be column-vectors in operations with
  matrices.

  Most elementary functions such as sin(), cosh(), log(), abs(), ..., are
  available as operations on the individual elements of the vectors. Please
  see the individual functions for more details.
*/
template<class T>
class Vec {
 public:
  //! Constructor
  Vec() { init(); }
  //! Constructor
  explicit Vec(int size) { it_assert1(size>=0,"Negative size in Vec::Vec(int)"); init(); alloc(size); }
  //! Constructor
  Vec(const Vec<T> &v);
  //! Constructor
  Vec(const char *values) { init(); set(values); }
  //! Constructor
  Vec(const std::string &values) { init(); set(values); }
  //! Constructor taking a C-array as input. Copies all data.
  Vec(T *c_array, int size) { init(); alloc(size); memcpy(data, c_array, size*sizeof(T)); }
  //! Destructor
  ~Vec() { free(); }

  //! The size of the vector
  int length() const { return datasize; }
  //! The size of the vector
  int size() const { return datasize; }

  //! Set length of vector. if copy = true then keeping the old values
  void set_length(int size, bool copy=false) { set_size(size,copy); }
  //! Set length of vector. if copy = true then keeping the old values
  void set_size(int size, bool copy=false);
  //! Set the vector to the all zero vector
  void zeros() { for (int i=0; i<datasize; i++) {data[i]=T(0);} }
  //! Set the vector to the all zero vector
  void clear() { zeros(); }
  //! Set the vector to the all one vector
  void ones() { for (int i=0; i<datasize; i++) {data[i]=T(1);} }
  //! Set the vector equal to the values in the \c str string
  bool set(const char *str);
  //! Set the vector equal to the values in the \c str string
  bool set(const std::string &str);

  //! C-style index operator. First element is 0
  T operator[](int i) const { it_assert0(i>=0&&i<datasize, "operator[]"); return data[i]; }
  //! Index operator. First element is 0
  T operator()(int i) const { it_assert0(i>=0&&i<datasize, "operator()"); return data[i]; }
  //! C-style index operator. First element is 0
  T &operator[](int i) { it_assert0(i>=0&&i<datasize, "operator[]"); return data[i]; }
  //! Index operator. First element is 0
  T &operator()(int i) { it_assert0(i>=0&&i<datasize, "operator()"); return data[i]; }
  //! Sub-vector with elements from \c i1 to \c i2. Index -1 indicates the last element.
  const Vec<T> operator()(int i1, int i2) const;
  //! Sub-vector where the elements are given by the list \c indexlist
  const Vec<T> operator()(const Vec<int> &indexlist) const;

  //! Addition of vector
  void operator+=(const Vec<T> &v);
  //! Addition of scalar
  void operator+=(T t) { for (int i=0;i<datasize;i++) data[i]+=t; }
  //! Addition of two vectors
  friend Vec<T> operator+TEMPLATE_FUN(const Vec<T> &v1, const Vec<T> &v2);
  //! Addition of a vector and a scalar
  friend Vec<T> operator+TEMPLATE_FUN(const Vec<T> &v, T t);
  //! Addition of a scalar and a vector
  friend Vec<T> operator+TEMPLATE_FUN(T t, const Vec<T> &v);
  
  //! Subtraction of vector
  void operator-=(const Vec<T> &v);
  //! Subtraction of scalar
  void operator-=(T t) { for (int i=0;i<datasize;i++) data[i]-=t; }
  //! Subtraction of \c v2 from \c v1
  friend Vec<T> operator-TEMPLATE_FUN(const Vec<T> &v1, const Vec<T> &v2);
  //! Subtraction of scalar from vector
  friend Vec<T> operator-TEMPLATE_FUN(const Vec<T> &v, T t);
  //! Sutraction of vector from scalar
  friend Vec<T> operator-TEMPLATE_FUN(T t, const Vec<T> &v);
  //! Negation of vector
  friend Vec<T> operator-TEMPLATE_FUN(const Vec<T> &v);
  
  //! Multiply with a scalar
  void operator*=(T t) { for (int i=0;i<datasize;i++) data[i] *= t; }
  //! Inner (dot) product
  friend T operator*TEMPLATE_FUN(const Vec<T> &v1, const Vec<T> &v2);
  //! Inner (dot) product
  friend T dot TEMPLATE_FUN(const Vec<T> &v1, const Vec<T> &v2);
  //! Outer product of two vectors v1 and v2
  friend Mat<T> outer_product TEMPLATE_FUN(const Vec<T> &v1, const Vec<T> &v2);
  //! Elementwise multiplication of vector and scalar
  friend Vec<T> operator*TEMPLATE_FUN(const Vec<T> &v, T t);
  //! Elementwise multiplication of vector and scalar 
  friend Vec<T> operator*TEMPLATE_FUN(T t, const Vec<T> &v);
  //! Elementwise multiplication
  friend Vec<T> elem_mult TEMPLATE_FUN(const Vec<T> &v1, const Vec<T> &v2);
  //! Elementwise multiplication of three vectors
  friend Vec<T> elem_mult TEMPLATE_FUN(const Vec<T> &v1, const Vec<T> &v2, const Vec<T> &v3);  
  //! Elementwise multiplication of four vectors
  friend Vec<T> elem_mult TEMPLATE_FUN(const Vec<T> &v1, const Vec<T> &v2, const Vec<T> &v3, const Vec<T> &v4);

  //! Elementwise division
  void operator/=(T t) { for (int i=0;i<datasize;i++) data[i]/=t; }
  //! Elementwise division
  friend Vec<T> operator/TEMPLATE_FUN(const Vec<T> &v, T t);
  //! Elementwise division
  friend Vec<T> operator/TEMPLATE_FUN(const T t, const Vec<T> &v);
  //! Elementwise division
  void operator/=(const Vec<T> &v);
  //! Elementwise division
  friend Vec<T> elem_div TEMPLATE_FUN(const Vec<T> &v1, const Vec<T> &v2);
  //! Elementwise division
  friend Vec<T> elem_div TEMPLATE_FUN(const T t, const Vec<T> &v);
  
  //! Get the elements in the vector where \c binlist is \c 1
  Vec<T> get(const Vec<bin> &binlist) const;
  //! Get the right \c nr elements from the vector
  Vec<T> right(int nr) const;
  //! Get the left \c nr elements from the vector
  Vec<T> left(int nr) const;
  //! Get the middle part of vector from \c start including \c nr elements
  Vec<T> mid(int start, int nr) const;
  //! Split the vector into two parts at element \c pos. Return the first part and keep the second.
  Vec<T> split(int pos);
  //! Shift in element \c In at position 0 \c n times
  void shift_right(T In, int n=1);
  //! Shift in vector \c In at position 0
  void shift_right(const Vec<T> &In);
  //! Shift out the \c n left elements and a the same time shift in the element \c at last position \c n times
  void shift_left(T In, int n=1);
  //! Shift in vector \c In at last position
  void shift_left(const Vec<T> &In);

  //! Append element \c a to the end of the vector \c v
  friend Vec<T> concat TEMPLATE_FUN(const Vec<T> &v, const T a);
  //! Concat element \c a to the beginning of the vector \c v
  friend Vec<T> concat TEMPLATE_FUN(const T a, const Vec<T> &v);
  //! Concat vectors \c v1 and \c v2
  friend Vec<T> concat TEMPLATE_FUN(const Vec<T> &v1,const Vec<T> &v2);
  //! Concat vectors \c v1, \c v2 and \c v3
  friend Vec<T> concat TEMPLATE_FUN(const Vec<T> &v1, const Vec<T> &v2, const Vec<T> &v3);

  //! Set subvector defined by indicies i1 to i2 to vector v
  void set_subvector(int i1, int i2, const Vec<T> &v);
  //! Set subvector defined by indicies i1 to i2 to constant t
  void set_subvector(int i1, int i2, const T t);
  //! Replace the elements from \c pos by the vector \c v
  void replace_mid(int pos, const Vec<T> &v);
  //! Delete element number \c index
  void del(int index);
  //! Insert element \c in at \c index
  void ins(int index, T in);
  //! Insert vector \c in at \c index
  void ins(int index, const Vec<T> &in);

  //! Assign all elements in vector to \c t
  void operator=(T t) { for (int i=0;i<datasize;i++) data[i] = t; }
  //! Assign vector the value and length of \c v
  void operator=(const Vec<T> &v);
  //! Assign vector equal to the 1-dimensional matrix \c m
  void operator=(const Mat<T> &m);
  //! Assign vector the values in the string \c values
  void operator=(const char *values) { set(values); }

  //! Elementwise equal to the scalar
  Vec<bin> operator==(const T value);
  //! Elementwise not-equal to the scalar
  Vec<bin> operator!=(const T value);
  //! Elementwise less than the scalar
  Vec<bin> operator<(const T value);
  //! Elementwise less than and equal to the scalar
  Vec<bin> operator<=(const T value);
  //! Elementwise greater than the scalar
  Vec<bin> operator>(const T value);
  //! Elementwise greater than and equal to the scalar
  Vec<bin> operator>=(const T value);

  //! Compare two vectors. False if wrong sizes or different values
  bool operator==(const Vec<T> &v) const;
  //! Compare two vectors. True if different
  bool operator!=(const Vec<T> &v) const;

  //! Index operator without boundary check. Not recommended to use.
  T &_elem(int i) { return data[i]; }
  //! Index operator without boundary check. Not recommended to use.
  T _elem(int i) const { return data[i]; }

  //! Get the pointer to the internal structure. Not recommended to use.
  T *_data() { return data; }

  //! Get the pointer to the internal structure. Not recommended to use.
  const T *_data() const { return data; }
    
 protected:
  //! Allocate storage for a vector of length \c size.
  void alloc(int size)
    {
      if ( datasize == size ) return;

      free();  // Free memory (if any allocated)
      if (size == 0) return;
    
      data = new T[size];
      datasize=size;
      it_assert1(data, "Out of memory in Vec::alloc()");
    }

  //! Free the storage space allocated by the vector
  void free() { delete [] data;  init(); }
   
  //! The current number of elements in the vector
  int datasize;
  //! A pointer to the data area
  T *data;

 private:
  void init() { data = 0; datasize = 0; }
};

/*!
  \relates Vec
  \brief Stream output of vector
*/
template <class T>
std::ostream &operator<<(std::ostream& os, const Vec<T> &v);

/*! 
  \relates Vec
  \brief Stream input of vector
*/
template <class T>
std::istream &operator>>(std::istream& is, Vec<T> &v);


/*! 
  \relates Vec
  \brief Definition of double vector type
*/
typedef Vec<double> vec;

/*! 
  \relates Vec
  \brief Definition of double_complex vector type
*/
typedef Vec<double_complex> cvec;

/*! 
  \relates Vec
  \brief Definition of integer vector type
*/
typedef Vec<int> ivec;


/*! 
  \relates Vec
  \brief Definition of long_long vector type
*/
typedef Vec<long_long> llvec;

/*! 
  \relates Vec
  \brief Definition of short vector type
*/
typedef Vec<short int> svec;

/*! 
  \relates Vec
  \brief Definition of binary vector type
*/
typedef Vec<bin> bvec;

/*------------------ Inline definitions ----------------------------*/

template<class T> inline
const Vec<T> Vec<T>::operator()(int i1, int i2) const
{
  if (i1 == -1)	i1 = datasize-1;
  if (i2 == -1) i2 = datasize-1;

  it_assert1(i1>=0 && i2>=0 && i1<datasize && i2<datasize, "Vec<T>::operator()(i1,i2): indicies out of range");
  it_assert1(i2>=i1, "Vec<T>::op(i1,i2): i2 >= i1 necessary");

   Vec<T> s(i2-i1+1);
   memcpy(s.data, data+i1, s.datasize*sizeof(T));

   return s;
}

#endif // __vec_h
