/*---------------------------------------------------------------------------*
 *                                   IT++			             *
 *---------------------------------------------------------------------------*
 * Copyright (c) 1995-2001 by Tony Ottosson, Thomas Eriksson, Pl Frenger,   *
 * Tobias Ringstrm, and Jonas Samuelsson.                                   *
 *                                                                           *
 * Permission to use, copy, modify, and distribute this software and its     *
 * documentation under the terms of the GNU General Public License is hereby *
 * granted. No representations are made about the suitability of this        *
 * software for any purpose. It is provided "as is" without expressed or     *
 * implied warranty. See the GNU General Public License for more details.    *
 *---------------------------------------------------------------------------*/

/*! 
  \file 
  \brief Definition of Communication Channel classes and functions 
  \author Pl Frenger

  1.9

  2003/01/04 00:21:55
*/
 
#ifndef __channel_h
#define __channel_h

#include "base/vec.h"
#include "base/mat.h"
#include "base/random.h"

/*! \defgroup channels Communication Channels
 */
  
/*! 
  \relates Rayleigh_Channel
  \ingroup channels 
  \brief Jakes spectrum filter

  Function that generates the taps in the Jake-filter.
  order is the number of taps in the filter.
  \a NormFdopp is the normalized doppler frequency, i.e. \a NormFDopp = \a Fd * \a Ts, where
  \a Fd is the acctual Doppler frequency and \a Ts is the sampling interval.
  Returns a vector containing the filter taps of the Jake-filter.
 */
vec jake_filter(double NormFDopp, int order = 100);

/*! 
  \ingroup channels
  \brief A Binary Symetric Channel with crossover probability p.

  Input and output are of type \c bvec with 0 and 1.
  Example:
  \code
  #include "itcomm.h"

  int main() {
    //Initiate the BSC with cross-over probability 0.1
    BSC bsc(0.1);

    bvec transmitted_bits = randb(100);
    bvec received_bits = bsc(transmitted_bits);
  }
  \endcode
*/
class BSC  {
 public:
  //! Class constructor. Sets the error probability to p
  BSC(double in_p=0.0) : u(0.0, 1.0) { p = in_p; };
  //! Set crossover (bit error) probability
  void set_prob(double in_p) { p = in_p; };
  //! Get crossover (bit error) probability
  double get_prob() { return p; };
  //! Feed \c input through the BSC channel
  bvec operator()(const bvec &input);
 protected:
 private:
  Uniform_RNG u;
  double p;
};

/*! 
  \ingroup channels
  \brief An abstract channel class
  \author Pl Frenger
*/
class Channel {
public:
  //! Virtual channel operator
  virtual cvec operator()(const cvec &input) = 0;
  //! Set the noise variance
  void set_noise(double noisevar) { awgnvar = noisevar; n.setup(0.0, awgnvar); sigma = sqrt(noisevar); };
  //! Get the noise variance
  double get_noise() { return awgnvar; };
 protected:
  //! ACTION: Add documentation for this protected member
  Channel(double noisevar = 0.0) : n(0.0,noisevar) { awgnvar = noisevar; sigma = sqrt(noisevar); };
  //! Protected Normal Random Number Generator
  Normal_RNG n;
  //! Variance of the AWGN
  double awgnvar;
  //! Standard deviation of the AWGN
  double sigma;
};

/*! 
  \ingroup channels
  \brief Ordinary AWGN Channel for cvec or vec inputs and outputs.
  
  Example:
  \code
  #include "itcomm.h"

  int main() {
  
    //Initiate the AWGN_Channel class
    double noisevar = 0.1;
    AWGN_Channel awgn_channel(noisevar);

    //Initiate a QPSK-modulator, and generate the transmitted signal
    QPSK qpsk;
    bvec transmitted_bits = randb(20);
    cvec transmitted_signal = qpsk.modulate_bits(transmitted_bits);

    //Usage of the member operator ()
    cvec received_signal = awgn_channel(transmitted_signal);

    //Demodulate the bits
    bvec received_bits = qpsk.demodulate_bits(received_signal);
  }
  \endcode
*/
class AWGN_Channel : public Channel {
public:
  //! Class constructor. Sets the noise variance.
  AWGN_Channel(double noisevar = 0.0) : Channel(noisevar) {};
  //! Feed the complex input \c input through the complex AWGN channel
  cvec operator()(const cvec &input);
  //! Feed the input \c through the AWGN channel
  vec operator()(const vec &input);
};

/*! 
  \ingroup channels
  \brief Rice fading channel for cvec or vec inputs and outputs. Perfect interleaving.

  Example:
  \code
  #include "itcomm.h"

  int main() {
    //The variance of the AWGN
    double noisevar = 0.1;
    //The ratio of the direct path and the indirect path in dB.
    double gamma = 0.0;
    //Initiate the class
    Rice_Channel rice_channel(noisevar,gamma);

    cvec transmitted_signal;
    //
    // Your code here generates the transmitted signal
    //
    
    cvec channel;
    //The used channel values is returned in the 'channel' vector
    cvec received_signal = rice_channel(transmitted_signal,channel);    
  }
  \endcode
*/
class Rice_Channel : public Channel {
 public:
  //! Class constructor. \a ingamma is the ratio of the direct path and the indirect path in dB.  The same seed will be used for both random sources. If the seed is zero a time-depending seed will be drawn.
  Rice_Channel(double noisevar = 0.0, double ingamma = 0.0) : Channel(noisevar), g(0, 0.25) { gamma = ingamma; };
  //! Feed the complex \c input through the complex Rice channel
  cvec operator()(const cvec &input);
  //! Feed the complex \c input through the complex Rice channel and also give the channel gains
  cvec operator()(const cvec &input, cvec &chan);
  //! Feed the \c input through the Rice channel
  vec operator()(const vec &input);
  //! Feed the \c input through the Rice channel and also give the channel gains
  vec operator()(const vec &input, vec &chan);
  //! Set the \a gamma value
  void set_gamma(double ingamma) { gamma = ingamma; };
  //! Get the \a gamma value
  double get_gamma() { return gamma; };

 protected:

 private:
  Normal_RNG g;
  double gamma; //Ratio of direct path and indirect path in dB.
};



/*! 
  \ingroup channels
  \brief Flat Rayleigh fading channel

  Flat Rayleigh fading channel for \c cvec or \c vec inputs and outputs, with or without doppler.
  AWGN of variance \a noisevar is added to the output.
  <ul>
  <li> \a normdoppler = 0 gives perfect interleaving. </li>
  <li> \a normdoppler = \a Fd * \a Ts > 0 gives correlated fading.</li>
  <li> By default the \c ifft algoritm is used to generate the fading process.</li>
  <li> If, however \a jakefilterorder > 0 the jake filter algoritm is used.
  \a jakefiltorder is the number of taps used to generate the fading process.
  \a upsamplingfactor > 1 can be used to generate slow fading without using an extremly long Jake-filter.
  Linear interpolation between fading samples is used together with jake-filtering.</li>
  </ul>
  Example:
  \code
  #include "itcomm.h"

  int main() {
    double noisevar = 0.05;
    double normdoppler = 0.1;
    
    //Initiate the class
    Rayleigh_Channel rayleigh_channel(noisevar,normdoppler);

    cvec transmitted_signal;
    //
    // Your code here generates the transmitted signal
    //
    
    cvec channel;
    //The used channel values is returned in the 'channel' vector
    cvec received_signal = rayleigh_channel(transmitted_signal,channel);    
  }
  \endcode
*/
class Rayleigh_Channel : public Channel {
 public:
  //! Consructor
  Rayleigh_Channel(double noisevar = 0.0, double normdoppler = 0.0, int jakefilterorder = 0, int upsamplingfactor = 1); 
  //! Feed the complex \a input through the Complex Rayleigh channel
  cvec operator()(const cvec &input);
  //! Feed the complex \a input through the Complex Raleigh channel and also give the the channe gains
  cvec operator()(const cvec &input, cvec &chan);
  //! Feed the \a input through the Rayleigh channel
  vec operator()(const vec &input);
  //! Feed the \a input through the Rayliegh channel and also give the channel gains
  vec operator()(const vec &input, vec &chan);
  //! Set normalized doppler frequency
  void set_doppler(double innormdopp) { normdopp = innormdopp; };
  //! Set the order of the Jake-filter for generating the fading process
  void set_filter_order(int injfiltord) { jfiltord = injfiltord; };
  //! Set the upsampling factor used for generating the fading process
  void set_up_sampling(int inupsamp) { upsamp = inupsamp; };
  //! Get the normalized doppler frequency
  double get_doppler(void) { return normdopp; };
  //! Get the filter order
  int get_filter_order(void) { return jfiltord; };
  //! Get the upsampling factor
  int get_up_sampling() { return upsamp; };
  //! Creates cvec with channel values. Useful for later combining
  void create_channel(int no_samples, cvec &channel);
  //! Filter the input through the Rayleigh channel. Not adding AWGN!
  cvec filter(const cvec &channel, const cvec &input);

 protected:

 private:
  double normdopp;
  int jfiltord;
  int upsamp;
};
 
/*! 
  \ingroup channels
  \brief Multipath Rayleigh fading Channel

  Frequency selective Multipath independent Rayleigh fading channel according to Jakes model.
  The fading process are generated by the \c ifft algorithm.
  <ul>
  <li> If \a delayvect = "0" it is assumed that delays are sample spaced \a 0:nopaths-1. </li>
  <li> If \a powervectdb = "0" it is assumed that all paths have equal power. </li>
  </ul>
  Example:
  \code
  #include "itcomm.h"

  int main() {
    double noisevar = 0.05; 
    double normdoppler = 0.1;
    short nopaths = 2;
    ivec delayvect = "0 7";
    vec powervecdb = "0 -3";
    
    //Initiate the class
    Multipath_Rayleigh_Channel multipath_rayleigh_channel(noisevar,normdoppler,nopaths,delayvect,powervecdb);

    cvec transmitted_signal;
    //
    // Your code here generates the transmitted signal
    //
    
    cmat channel;
    //The used channel values for each channel path is returned in the 'channel' matrix
    cvec received_signal = multipath_rayleigh_channel(transmitted_signal,channel);    
  }
  \endcode
*/
class Multipath_Rayleigh_Channel  : public Channel {
 public:
  //!  If the seed is zero a time-depending seed will be drawn.
  Multipath_Rayleigh_Channel(double inawgnvar = 0.0, 
			     double innormdopp = 0.1, 
			     short innopaths = 2, 
			     ivec indelayvect = "0", 
			     vec inpowervecdb = "0");
  //! Destructor
  virtual ~Multipath_Rayleigh_Channel() { }
  //! Feed the complex \a input through the Multipath channel and also give the channel gains
  cvec operator()(const cvec &input, cmat &chan);
  //! Feed the complex \a input through the Multipath channel
  cvec operator()(const cvec &input);
  //! Set the normalized doppler frequency
  void set_doppler(double innormdopp) { normdopp = innormdopp; };
  //! Set the number of discrete paths
  void set_no_paths(short innopaths) { nopaths = innopaths; };
  //! Set the delays for the discrete paths. Should be in samples
  void set_delay_vect(ivec indelayvect) { delvec = indelayvect; };
  //! Set the power for each of the paths. Given in dB.
  void set_power_vec_db(vec inpowervecdb) { powvecdb = inpowervecdb; };
  //! Get the normalized doppler frequency
  double get_doppler() {return normdopp;};
  //! Get the number of discrete paths used in the channel
  short get_no_paths() {return nopaths;};
  //! Get the delays for the discrete paths
  ivec get_delay_vect() {return delvec;};
  //! Get the power for the discrete paths
  vec get_power_vec_db() {return powvecdb;};
  //! Creates cmat with channel values for all paths. Useful for later combining
  void create_channel(int no_samples, cmat &channel);
  //! Filter the input through the multipath channel. Not adding AWGN!
  cvec filter(const cmat &channel, const cvec &input);
 private: 
  double normdopp;
  short  nopaths;
  ivec   delvec;
  vec   powvecdb;
};

#endif // __channel_h
