/*---------------------------------------------------------------------------*
 *                                   IT++			             *
 *---------------------------------------------------------------------------*
 * Copyright (c) 1995-2001 by Tony Ottosson, Thomas Eriksson, Pl Frenger,   *
 * Tobias Ringstrm, and Jonas Samuelsson.                                   *
 *                                                                           *
 * Permission to use, copy, modify, and distribute this software and its     *
 * documentation under the terms of the GNU General Public License is hereby *
 * granted. No representations are made about the suitability of this        *
 * software for any purpose. It is provided "as is" without expressed or     *
 * implied warranty. See the GNU General Public License for more details.    *
 *---------------------------------------------------------------------------*/

/*! 
  \file 
  \brief Definitions of interleaver classes
  \author Pl Frenger

  1.7

  2002/02/18 23:15:36
*/

#ifndef __interleave_h
#define __interleave_h

template<class T> class Vec;
template<class T> class Mat;

/*! \defgroup interl Interleavers
 */

/*! \ingroup interl
  \class Block_Interleaver comm/interleave.h
  \brief Block Interleaver Class.
  
  Data is written row-wise and read column-wise when interleaving.
  <h3>Example of use:</h3>
  \code
  BPSK bpsk;
  bvec bits = "0 1 1 0 0 0 1 1 1 1 0 0 1 0 0 1";
  vec symbols = bpsk.modulate_bits(bits);

  Block_Interleaver<double> block_interleaver(4,4);
  vec interleaved_symbols = block_interleaver.interleave(symbols);
  \endcode
  
*/
template <class T>
class Block_Interleaver {
public:
    //! Block_Interleaver constructor
    Block_Interleaver(void) {rows = 0; cols = 0;};
    //! Block_Interleaver constructor
    Block_Interleaver(int in_rows, int in_cols);
    //! Function for block interleaving. May add some zeros.
    Vec<T> interleave(const Vec<T> &input);
    //! Function for block interleaving. May add some zeros.
    void interleave(const Vec<T> &input, Vec<T> &output);
    //! Function for block deinterleaving. Removes additional zeros if \a keepzeros = 0.
    Vec<T> deinterleave(const Vec<T> &input, short keepzeros = 0 );
    //! Function for block deinterleaving. Removes additional zeros if \a keepzeros = 0.
    void deinterleave(const Vec<T> &input, Vec<T> &output, short keepzeros = 0 );
    //! Set the number of \a rows for block interleaving
    void set_rows(int in_rows) {rows = in_rows;};
    //! Set the number of \a columns for block interleaving
    void set_cols(int in_cols) {cols = in_cols;};
    //! Get the number of \a rows for block interleaving
    int get_rows(void) {return rows;};
    //! Get the number of \a columns for block interleaving    
    int get_cols(void) {return cols;};
private:
    int rows, cols, input_length;
};

/*! \ingroup interl
  \class Cross_Interleaver comm/interleave.h
  \brief Cross Interleaver Class.

  <h3> Example of use: </h3>
  \code
  BPSK bpsk;
  bvec bits = "0 1 1 0 0 0 1 1 1 1 0 0 1 0 0 1";
  vec symbols = bpsk.modulate_bits(bits);

  Cross_Interleaver<double> cross_interleaver(4);
  vec interleaved_symbols = cross_interleaver.interleave(symbols);
  \endcode
  
  <ul>
  <li> See S. B. Wicker, "Error control systems for digital communications and storage,"
  Prentice Hall 1995, p. 427 for details. </li>
  </ul>
*/
template <class T>
class Cross_Interleaver {
public:
    //! Cross_Interleaver constructor
    Cross_Interleaver(void) {order = 0;};
    //! Cross_Interleaver constructor
    Cross_Interleaver(int in_order);
    //! Function for cross interleaving. Adds some zeros.
    Vec<T> interleave(const Vec<T> &input);
    //! Function for cross interleaving. Adds some zeros.
    void interleave(const Vec<T> &input, Vec<T> &output);
    //! Function for cross deinterleaving. Removes aditional zeros if \a keepzeros = 0.
    Vec<T> deinterleave(const Vec<T> &input, short keepzeros = 0);
    //! Function for cross deinterleaving. Removes aditional zeros if \a keepzeros = 0.
    void deinterleave(const Vec<T> &input, Vec<T> &output, short keepzeros = 0);
    //! Set the \a order of the Cross Interleaver
    void set_order(int in_order); 
     //! Get the \a order of the Cross Interleaver
    int get_order(void) {return order;};
private:
    int order;
    int input_length;
    Mat<T> inter_matrix;
    Vec<T> tempvec, zerostemp;
};

/*! \ingroup interl
  \class Sequence_Interleaver comm/interleave.h
  \brief Sequence Interleaver Class

  <h3> Example of use: </h3>
  \code
  BPSK bpsk;
  bvec bits = "0 1 1 0 0 0 1 1 1 1 0 0 1 0 0 1";
  vec symbols = bpsk.modulate_bits(bits);

  Sequence_Interleaver<double> sequence_interleaver(16);
  sequence_interleaver.randomize_interleaver_sequence();
  vec interleaved_symbols = sequence_snterleaver.interleave(symbols);
  \endcode
  
*/
template <class T>
class Sequence_Interleaver {
public:
    //! Sequence_Interleaver constructor.
    Sequence_Interleaver(void) {interleaver_depth = 0;};
    /*! 
      \brief Sequence_Interleaver constructor.

      Chooses a random sequence of length \a in_interleaver_depth for interleaving.
    */
    Sequence_Interleaver(int in_interleaver_depth);
    /*! 
      \brief Sequence_Interleaver constructor.

      Uses the \a in_interleaver_sequence for interleaving.
    */
    Sequence_Interleaver(ivec in_interleaver_sequence);
    //! Function for sequence interleaving. May add some zeros.
    Vec<T> interleave(const Vec<T> &input);
    //! Function for sequence interleaving. May add some zeros.
    void interleave(const Vec<T> &input, Vec<T> &output);
    //! Function for sequence deinterleaving. Removes additional zeros if \a keepzeros = 0.
    Vec<T> deinterleave(const Vec<T> &input, short keepzeros = 0 );
    //! Function for sequence deinterleaving. Removes additional zeros if \a keepzeros = 0.
    void deinterleave(const Vec<T> &input, Vec<T> &output, short keepzeros = 0 );
    //! Generate a new random sequence for interleaving.
    void randomize_interleaver_sequence();
    //! Returns the interleaver sequence presently used.
    ivec get_interleaver_sequence();
    //! Set the interleaver sequence to be used.
    void set_interleaver_sequence(ivec in_interleaver_sequence);
    //! Set the length of the interleaver sequence to be used.
    void set_interleaver_depth(int in_interleaver_depth) { interleaver_depth = in_interleaver_depth; };
    //! Get the length of the interleaver sequence presently used.
    int get_interleaver_depth(void) { return interleaver_depth; };
private:
    ivec interleaver_sequence;
    int interleaver_depth, input_length;
};

#endif // __interleave_h
