/*---------------------------------------------------------------------------*
 *                                   IT++			             *
 *---------------------------------------------------------------------------*
 * Copyright (c) 1995-2002 by Tony Ottosson, Thomas Eriksson, Pl Frenger,   *
 * Tobias Ringstrm, and Jonas Samuelsson.                                   *
 *                                                                           *
 * Permission to use, copy, modify, and distribute this software and its     *
 * documentation under the terms of the GNU General Public License is hereby *
 * granted. No representations are made about the suitability of this        *
 * software for any purpose. It is provided "as is" without expressed or     *
 * implied warranty. See the GNU General Public License for more details.    *
 *---------------------------------------------------------------------------*/

/*! 
  \file 
  \brief Definitions of pulse shaping classes
  \author Tony Ottosson and Hkan Eriksson

  1.8

  2003/01/04 00:21:55
*/

#ifndef __pulse_shape_h
#define __pulse_shape_h


#include "base/matfunc.h"
#include "base/filter.h"
#include "base/vec.h"

/*! 
  \brief General FIR Pulse Shape

  Upsamples and shapes symbols according to a given FIR filter. Observe that since the shaping
  is done with a FIR filter, the first samples in the output are zero or small before the memory
  of the filter is filled.
  
  The class is templated as follows: 
  <ul>
  <li> \c T1 is the type of the input samples</li>
  <li> \c T2 is the type of the filter coefficients</li>
  <li> \c T3 is the type of the output samples</li>
  </ul>

  An example of usage is:
  
  \code
  #include "itcomm.h"

     vec filter_resonse;
     filter_response ="0.7 0.3 0.6";
     Pulse_Shape<double,double,double> shaper(filter_response, 4);
     BPSK bpsk;
     vec symbols, samples;
    
     symbols = bpsk.modulate_bits(randb(20));
     samples = shaper.shape_symbols(symbols);
  \endcode
*/
template<class T1, class T2, class T3>
class Pulse_Shape {
 public:
  //! Constructor
  Pulse_Shape();
  //! Constructor
  Pulse_Shape(const Vec<T2> &impulse_response, int over_sampling_factor);
  //! Destructor
  virtual ~Pulse_Shape() { }
  /*! 
    \brief Set the general impulse response of the FIR filter
    
    Observe that the pulse shape must have a duration of an integer number 
    of symbols. Thus the length of the impulse response-1 modulo over sampling
    is an integer.
  */
  void set_pulse_shape(const Vec<T2> &impulse_response, int over_sample_factor);
  //! Get the pulse shape
  Vec<T2> get_pulse_shape(void);
  //! Get the over sampling factor
  int get_over_sample_factor();
  //! Get the length of the pulse in number of symbols
  int get_pulse_length();
  //! Get the length of the internal FIR filter
  int get_filter_length();
  
  //! Shape the input symbols
  void shape_symbols(const Vec<T1> &input, Vec<T3> &output);
  //! Shape the input symbols
  Vec<T3> shape_symbols(const Vec<T1> &input);
  //! Shape the input samples (i.e. no over sampling)
  void shape_samples(const Vec<T1> &input, Vec<T3> &output);
  //! Shape the input samples(i.e. no up sampling)
  Vec<T3> shape_samples(const Vec<T1> &input);

  //! Clear internal states
  void clear(void);

 protected:
  //! The impulse resounse of the pulse shaping filter
  Vec<T2> _filter_impulse_response;
  //! The pulse shaping filter
  MA_Filter<T1,T2,T3> _filter;
  //! Length in symbols
  int _pulse_length;
  //! Samples per symbol = over sampling factor
  int _over_sample_factor; 
  //! Ensures that setup is called before any other member function
  bool _setup_done;
};

/*! 
  \brief Raised-Cosine (RC) Pulse Shaper

  Upsamples and shapes symbols as Raised-Cosine pulses with a given roll-off factor \f$ \alpha \f$.
  The Raised-Cosine pulse shape is defined as:
  \f[
  p(t) = \frac{\sin(\pi t / T)}{\pi t / T} \frac{\cos(\alpha \pi t / T)}{1 - (2 \alpha t / T)^2}
  \f]
  For more details see e.g. Lee & Messerschmitt, p. 190. Observe that the shaping is done
  with a FIR filter where the size is given by \a filter_length * \a over_sample_factor + 1.
  The first samples in the output will therefore be zero or small before the memory of the
  filter is filled.

  The class is templated as follows: \c T1 is the type of the input and the output samples.  
  An example of usage is:
  
  \code
  #include "itcomm.h"
  
    Raised_Cosine<double> rc(0.5,6,8);
    BPSK bpsk;
    vec symbols, samples;
    
    symbols = bpsk.modulate_bits(randb(20));
    samples = rc.shape_symbols(symbols);
  \endcode
*/
template<class T1>
class Raised_Cosine : public Pulse_Shape<T1,double,T1> {
 public:
  //! Constructor
  Raised_Cosine() { }
  //! Constructor
  Raised_Cosine(double roll_off, int filter_length = 6, int over_sample_factor = 8);
  //! Destructor
  virtual ~Raised_Cosine() { }
  //! Set pulse shape. Roll-off factor between 0 and 1. filter_length even
  void set_pulse_shape(double roll_off_factor, int filter_length = 6, int over_sample_factor = 8);
  //! Get the roll-off factor
  double get_roll_off(void);
    
 protected:
  //! The roll off factor (i.e. alpha)
  double _roll_off_factor;
};

/*! 
  \brief (Square) Root Raised-Cosine (RC) Pulse Shaper

  Upsamples and shapes symbols as Square Root Raised-Cosine pulses with a given
  roll-off factor \f$ \alpha \f$ (zero is not allowed. Use Raised-Cosine instead).
  The Root Raised -Cosine pulse shape is defined as:
  \f[
  p(t) = \frac{4 \alpha}{\pi \sqrt{T}}
  \frac{\cos((1+\alpha)\pi t / T) + T \sin((1-\alpha)\pi t / T) / (4 \alpha t) }{1 - (4 \pi t / T)^2}
  \f]
  It is called square root raised-cosine since it is defined as the square root of the raised-cosine
  pulse in the frequency domain. Thus with a transmitter pulse shape of Root raised-cosine and a receiver
  filter that is root raised cosine, the overall response will be a Raised-Cosine.

  For more details see e.g. Lee & Messerschmitt, p. 228. Observe that the shaping is done with
  a FIR filter where the size is given by \a filter_length * \a over_sample_factor + 1. The first
  samples in the output will therefore be zero or small before the memory of the filter is filled.
  
  The class is templated as follows: \c T1 is the type of the input and the output samples.  
  An example of usage is:
  
  \code
  #include "itcomm.h"
  
    Root_Raised_Cosine<double> rrc(0.5,6,8);
    BPSK bpsk;
    vec symbols, samples;
    
    symbols = bpsk.modulate_bits(randb(20));
    samples = rrc.shape_symbols(symbols);
  \endcode
*/
template<class T1>
class Root_Raised_Cosine : public Pulse_Shape<T1,double,T1> {
 public:
  //! Constructor
  Root_Raised_Cosine() { }
  //! Constructor
  Root_Raised_Cosine(double roll_off_factor, int filter_length = 6, int over_sample_factor = 8);
  //! Destructor
  virtual ~Root_Raised_Cosine() { }
  //! Set pulse shape. Roll-off factor between 0 and 1. filter_length even
  void set_pulse_shape(double roll_off_factor, int filter_length = 6, int over_sample_factor = 8);
  //! Get the Roll-off factor
  double get_roll_off(void);

 protected:

  //! The roll off factor (i.e. alpha)
  double _roll_off_factor;
};

#endif // __pulse_shape_h
