/*---------------------------------------------------------------------------*
 *                                   IT++			             *
 *---------------------------------------------------------------------------*
 * Copyright (c) 1995-2001 by Tony Ottosson, Thomas Eriksson, Pl Frenger,   *
 * Tobias Ringstrm, and Jonas Samuelsson.                                   *
 *                                                                           *
 * Permission to use, copy, modify, and distribute this software and its     *
 * documentation under the terms of the GNU General Public License is hereby *
 * granted. No representations are made about the suitability of this        *
 * software for any purpose. It is provided "as is" without expressed or     *
 * implied warranty. See the GNU General Public License for more details.    *
 *---------------------------------------------------------------------------*/

/*! 
  \file 
  \brief Definitions of audio device classes
  \author Tobias Ringstrm

  1.8

  2003/01/04 00:21:55
*/

#ifndef __audiodev_h
#define __audiodev_h

#include "itconfig.h"
#include "base/vec.h"

// This is neccesary because of a limitation in gcc 2.7.2 
//#define PROTECTED public 

/*! 
  \ingroup audio
  \brief Base class for Audio I/O Classes.

  Since this is a base class you probably do not want to use this class directly. Use the inherited classes instead.
  Audio classes are only implemented for Linux, Solaris, and Windows. 
*/
class Audio_Basic_Device {
public:
  /*! 
    \brief Definition of the \c open_mode data type

    <ul>
    <li> \c mode_rd stands for "Read Mode" </li>
    <li> \c mode_wr stands for "Write Mode" </li>
    <li> \c mode_rdwr stands for "Read and Write Mode" </li>
    </ul>
  */
  enum open_mode { mode_rd, mode_wr, mode_rdwr };
	
  /*! 
    \brief Class Constructor

  */
  Audio_Basic_Device();
  
  //! Class Destructor
  virtual ~Audio_Basic_Device();
  
  /*! 
    \brief Open an audio device 

    \param mode Set the read/write mode. Possible values are \c mode_rd (read), \c mode_wr (write), and \c mode_rdwr (read and write).
  */
  bool open(open_mode mode);

  //! Returns \c true if the audio device is open and \c false otherwise.
  bool is_open() const { return isOpen; }

  //! Close the audio device
  void close();

  /*! 
    \brief Read \c n signed char values of raw data. 

    You probably want to use the record() function instead. This function is not implemented in Windows.
  */
  int read(signed char *p, int n);

  /*! 
    \brief Read \c n short values of raw data. 

    You probably want to use the record() function instead. This function is not implemented in Windows.
  */
  int read(short *p, int n);

  //! Write \c n signed char values of raw data. You probably want to use the play() function instead!
  int write(const signed char *p, int n);

  //! Write \c n short values of raw data. You probably want to use the play() function instead!
  int write(const short *p, int n);
	
 protected:
  struct Device;
  //! ACTION: Document this protected member
  Device *dev;
  //! ACTION: Document this protected member
  bool isOpen;
};

/*! 
  \ingroup audio
  \brief Audio Output (Play) Class

  This audio class works in Linux, Solaris, and Windows only. In Linux and in Solaris the audio device used is \c /dev/audio.
*/
class Audio_Play_Device : virtual public Audio_Basic_Device { //class Audio_Play_Device : virtual PROTECTED Audio_Basic_Device

public:

  /*!
    \brief Class constructor and class setup function

    \param max_amplitude Define the maximum amplitude of the values to be played through this audio device.
    \param rate The sample rate in Hz.
    \param bits The number of bits per sample (default value is 16)
    \param stereo Defines if stereo or mono play back should be used. Default value is \c false (i.e. mono).
  */
  Audio_Play_Device(double max_amplitude, int rate, int bits=16, bool stereo=false);

  //! Class Destructor
  ~Audio_Play_Device();

  //! Returns the play sample rate in Hz used for this audio device.
  int get_play_rate();

  //! Sets the play sample rate in Hz to be used by this audio device. Returns \c true if successful.
  bool set_play_rate(int rate);
	
  //! Returns the number of bits per sample used for this audio device.
  int get_play_bits();

  /*! 
    \brief Sets the number of bits per sample to be used by this audio device. Returns \c true if successful.

    Possible values are 8 bits or 16 bits.
  */
  bool set_play_bits(int bits);
	
  //! Returns \c true if the audio device is configured for stereo play back and \c false otherwise.
  bool query_play_stereo();

  //! Configure this audio device for stereo play back. Returns \c true if successful.
  bool enable_play_stereo(bool on);

  //! Returns the buffersize used for this audio device
  int get_play_buffersize();

  /*! 
    \brief Sets the buffersize to be used by this audio device. Returns \c true if successful.
    
    This function is not implemented for Linux. As of now it always returns \c false in Linux.
  */
  bool set_play_buffersize(int size);

  //! Play a mono sound. Returns \c true if successful.
  bool play(const vec &mono);

  //! Play a stereo sound. Returns \c true if successful.
  bool play(const vec &left, const vec &right);

  /*! 
    \brief Stop the play of the sound.

    This function is not implemented in Linux or Windows. As of now it just generates an \c it_error that stops 
    the program in Linux or Windows (as well as the sound).
  */
  void stop_play();
  
  /*! 
    \brief Flush the audio device buffer

    This function is not implemented in Linux or Windows. As of now it just generates an \c it_error that stops 
    the program in Linux or Windows.
  */
  void flush();
    
protected:
    //! ACTION: ADD DOCUMENTATION FOR THIS PROTECTED MEMBER
    Audio_Play_Device(double max_aplitude);
    //! ACTION: ADD DOCUMENTATION FOR THIS PROTECTED MEMBER
    double play_max_ampl;

private:
    int play_rate, play_bits;
    bool play_stereo;
};

/*! 
  \ingroup audio
  \brief Audio Input (Record) Class

  This audio class works in Linux, Solaris, and Windows only. In Linux and in Solaris the audio device used is \c /dev/audio.
*/
class Audio_Rec_Device : virtual public Audio_Basic_Device { //class Audio_Rec_Device : virtual PROTECTED Audio_Basic_Device {
public:

  /*!
    \brief Class constructor and class setup function
    
    \param max_amplitude Define the maximum amplitude of the values to be recorded through this audio device.
    \param rate The sample rate in Hz.
    \param bits The number of bits per sample (default value is 16)
    \param stereo Defines if stereo or mono recording should be used. Default value is \c false (i.e. mono).
  */ 
  Audio_Rec_Device(double max_amplitude, int rate, int bits=16, bool stereo=false);
  
  //! Class Destructor
  ~Audio_Rec_Device();

  //! Returns the record sample rate in Hz used for this audio device.
  int get_rec_rate();
  
  //! Sets the record sample rate in Hz to be used by this audio device. Retuns \c true if successful.
  bool set_rec_rate(int rate);
	
  //! Returns the number of bits per sample used for this audio device.
  int get_rec_bits();
  
  /*! 
    \brief Sets the number of bits per sample to be used by this audio device. Returns \c true if successful.

    Possible values are 8 bits or 16 bits.
  */
  bool set_rec_bits(int bits);
	
  //! Returns \c true if the audio device is configured for stereo recording and \c false otherwise.
  bool query_rec_stereo();
  
  //! Configure this audio device for stereo recording. Returns \c true if successful.
  bool enable_rec_stereo(bool on);

  //! Returns the buffersize used for this audio device.
  int get_rec_buffersize();
  
  /*! 
    \brief Sets the buffersize to be used by this audio device. Returns \c true if successful.
    
    This function is not implemented in Linux or Windows. As of now it always returns \c false in Linux or Windows.
  */
  bool set_rec_buffersize(int size);
	
  //! Records \c n audio mono samples from this audio device.
  bool record(vec &mono, int n);
  
  //! Records \c n audio stereo samples from this audio device.
  bool record(vec &left, vec &right, int n);

  /*! 
    \brief Stop the recording of the sound.

    This function is not implemented in Linux or Windows. As of now it just generates an \c it_error that 
    stops the program in Linux or Windows (as well as the recording).
  */
  void stop_record();
    
protected:
    //! ACTION: Add documentation for this protected member
    Audio_Rec_Device(double max_aplitude);
    //! ACTION: Add documentation for this protected member
    double rec_max_ampl;

private:
    int rec_rate, rec_bits;
    bool rec_stereo;
};

/*! 
  \ingroup audio
  \brief Audio I/O (Play and Record) Class

  This audio class works in Linux, Solaris, and Windows only. In Linux and in Solaris the audio device used is \c /dev/audio.
*/
class Audio_IO_Device : public Audio_Play_Device, public Audio_Rec_Device {
public:

  /*!
    \brief Class constructor and class setup function
    
    \param max_amplitude Define the maximum amplitude of the values to be recorded or played through this audio device.
    \param rate The sample rate in Hz.
    \param bits The number of bits per sample (default value is 16)
    \param stereo Defines if stereo or mono play or recording should be used. Default value is \c false (i.e. mono).
  */ 
  Audio_IO_Device(double max_amplitude, int rate, int bits=16, bool stereo=false);

  //! Class Destructor
  ~Audio_IO_Device();

  //! Sets the record or play sample rate in Hz to be used by this audio device. Retuns \c true if successful.
  bool set_rate(int rate);

  /*! 
    \brief Sets the number of bits per sample to be used by this audio device. Returns \c true if successful.
    
    Possible values are 8 bits or 16 bits.
  */
  bool set_bits(int bits);

  //! Configure this audio device for stereo play or recording. Returns \c true if successful.
  bool enable_stereo(bool on);

  /*! 
    \brief Sets the buffersize to be used by this audio device. Returns \c true if successful.
    
    This function is not implemented in Linux. As of now it always returns \c false in Linux.
  */
  bool set_buffersize(int size);

  /*! 
    \brief Stop the recording or play back of the sound.

    This function is not implemented in Linux or Windows. As of now it just generates an \c it_error that 
    stops the program in Linux or Windows.
  */
  void stop();
};

#endif // __audiodev_h
